import re


class Endpoint:
    def __init__(self, url: str, **kwargs):
        self._url = url

        url_params = re.findall(r"{(\w*)}", self._url)

        if "" in url_params:
            raise ValueError("nameless format parameters not supported!")
        self._url_params = url_params
        self._query_params = [key for key in kwargs.keys() if key not in url_params]

    def __call__(self, **kwargs):
        for req_param in self._url_params:
            if req_param not in kwargs:
                raise ValueError(f'parameter "{req_param}" is required!')

        query_params = {
            key: value for key, value in kwargs.items() if key in self._query_params
        }

        return (self._url.format(**kwargs), query_params)



import json

from ..Deserializer import Deserializer


class DictionaryDeserializer(Deserializer):
    def deserialize(self, endpoint_name: str, method_name: str, data: str) -> dict:
        if not data:
            return {}
        return json.loads(data)



from typing import Set

from .. import BaseApi, NamedEndpoint
from .urls import MatchApiV4Urls


class MatchApiV4(NamedEndpoint):
    """
    This class wraps the Match-v4 endpoint calls provided by the Riot API.

    See https://developer.riotgames.com/api-methods/#match-v4 for more detailed information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new MatchApiV4 which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def by_id(self, region: str, match_id: int):
        """
        Get match by match ID

        :param string region: The region to execute this request on
        :param long match_id: The match ID.

        :returns: MatchDto
        """
        return self._request_endpoint(
            self.by_id.__name__, region, MatchApiV4Urls.by_id, match_id=match_id
        )

    def matchlist_by_account(
        self,
        region: str,
        encrypted_account_id: str,
        queue: Set[str] = None,
        begin_time: int = None,
        end_time: int = None,
        begin_index: int = None,
        end_index: int = None,
        season: Set[int] = None,
        champion: Set[int] = None,
    ):
        """
        Get matchlist for ranked games played on given account ID and platform ID
        and filtered using given filter parameters, if any

        A number of optional parameters are provided for filtering. It is up to the caller to
        ensure that the combination of filter parameters provided is valid for the requested
        account, otherwise, no matches may be returned.

        Note that if either beginIndex or endIndex are specified, then both must be specified and
        endIndex must be greater than beginIndex.

        If endTime is specified, but not beginTime, then beginTime is effectively the start of the
        account's match history.

        If beginTime is specified, but not endTime, then endTime is effectively the current time.

        Note that endTime should be greater than beginTime if both are specified, although there is
        no maximum limit on their range.

        :param string region:               The region to execute this request on
        :param string encrypted_account_id: The account ID.
        :param Set[int] queue:              Set of queue IDs for which to filtering matchlist.
        :param long begin_time:             The begin time to use for filtering matchlist specified
                                            as epoch milliseconds.
        :param long end_time:               The end time to use for filtering matchlist specified
                                            as epoch milliseconds.
        :param int begin_index:             The begin index to use for filtering matchlist.
        :param int end_index:               The end index to use for filtering matchlist.
        :param Set[int] season:             Set of season IDs for which to filtering matchlist.
        :param Set[int] champion:           Set of champion IDs for which to filtering matchlist.

        :returns: MatchlistDto
        """
        return self._request_endpoint(
            self.matchlist_by_account.__name__,
            region,
            MatchApiV4Urls.matchlist_by_account,
            encrypted_account_id=encrypted_account_id,
            queue=queue,
            beginTime=begin_time,
            endTime=end_time,
            beginIndex=begin_index,
            endIndex=end_index,
            season=season,
            champion=champion,
        )

    def timeline_by_match(self, region: str, match_id: int):
        """
        Get match timeline by match ID.

        Not all matches have timeline data.

        :param string region: The region to execute this request on
        :param long match_id: The match ID.

        :returns: MatchTimelineDto
        """
        return self._request_endpoint(
            self.timeline_by_match.__name__,
            region,
            MatchApiV4Urls.timeline_by_match,
            match_id=match_id,
        )



from .. import BaseApi, NamedEndpoint
from .urls import AccountApiUrls


class AccountApi(NamedEndpoint):
    """
    This class wraps the Account-v1 Api calls provided by the Riot API.

    See https://developer.riotgames.com/apis#account-v1 for more detailed information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new AccountApi which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def by_puuid(self, region: str, puuid: str):
        """
        Get account by puuid

        :returns: AcountDto
        """
        return self._request_endpoint(
            self.by_puuid.__name__, region, AccountApiUrls.by_puuid, puuid=puuid
        )

    def by_riot_id(self, region: str, game_name: str, tag_line: str):
        """
        Get account by riot id

        :returns: AccountDto
        """
        return self._request_endpoint(
            self.by_riot_id.__name__,
            region,
            AccountApiUrls.by_riot_id,
            game_name=game_name,
            tag_line=tag_line,
        )

    def active_shard(self, region: str, game: str, puuid: str):
        """
        Get active shard for a player

        :returns: ActiveShardDto
        """
        return self._request_endpoint(
            self.active_shard.__name__,
            region,
            AccountApiUrls.active_shard,
            game=game,
            puuid=puuid,
        )



import requests

from .RequestHandler import RequestHandler

ApiError = requests.HTTPError


class ThrowOnErrorHandler(RequestHandler):
    def after_request(
        self,
        region: str,
        endpoint_name: str,
        method_name: str,
        url: str,
        response: requests.Response,
    ) -> None:
        try:
            response.raise_for_status()
        except requests.HTTPError as err:
            raise ApiError(*err.args, request=err.request, response=err.response)



import datetime
import logging
import time

from requests import Response

from .RequestHandler import RequestHandler
from ..RateLimiter import RateLimiter

log = logging.getLogger(__name__)


class RateLimiterAdapter(RequestHandler):
    def __init__(self, limiter: RateLimiter):
        self._limiter = limiter

    def preview_request(
        self,
        region: str,
        endpoint_name: str,
        method_name: str,
        url: str,
        query_params: dict,
    ):
        """
        called before a request is processed.

        :param string region: the region of this request
        :param string endpoint_name: the name of the endpoint being requested
        :param string method_name: the name of the method being requested
        :param url: the URL that is being requested.
        :param query_params: dict: the parameters to the url that is being queried,
                                   e.g. ?key1=val&key2=val2
        """
        wait_until = self._limiter.wait_until(region, endpoint_name, method_name)

        if wait_until is not None and wait_until > datetime.datetime.now():
            to_wait = wait_until - datetime.datetime.now()

            log.info(
                "waiting for %s seconds...", to_wait.total_seconds(),
            )
            time.sleep(to_wait.total_seconds())

    def after_request(
        self,
        region: str,
        endpoint_name: str,
        method_name: str,
        url: str,
        response: Response,
    ) -> Response:
        """
        Called after a response is received and before it is returned to the user.

        :param string region: the region of this request
        :param string endpoint_name: the name of the endpoint that was requested
        :param string method_name: the name of the method that was requested
        :param url: The url that was requested
        :param response: the response received. This is a response from the Requests library
        """
        self._limiter.record_response(
            region, endpoint_name, method_name, response.status_code, response.headers
        )

        return response



from . import RequestHandler


class TypeCorrectorHandler(RequestHandler):
    """
    The TypeCorrector class is meant to correct any inconsistencies in the types
    of objects provided as query parameters.

    Currently this only involves changing boolean values into strings,
    as the API only accepts lower case booleans for some reason.
    """

    def preview_request(
        self,
        region: str,
        endpoint_name: str,
        method_name: str,
        url: str,
        query_params: dict,
    ):
        """
        called before a request is processed.

        :param string endpoint_name: the name of the endpoint being requested
        :param string method_name: the name of the method being requested
        :param url: the URL that is being requested.
        :param query_params: dict: the parameters to the url that is being queried,
                                   e.g. ?key1=val&key2=val2
        """
        if query_params is not None:
            for key, value in query_params.items():
                if isinstance(value, bool):
                    query_params[key] = str(value).lower()

                # check to see if we have a list/tuple, but not a string
                if (
                    not hasattr(value, "strip")
                    and hasattr(value, "__getitem__")
                    or hasattr(value, "__iter__")
                ):
                    for idx, val in enumerate(value):
                        if isinstance(val, bool):
                            value[idx] = str(val).lower()



from .ChampionApiUrls import ChampionApiV3Urls
from .ChampionMasteryApiUrls import ChampionMasteryApiV4Urls
from .ClashApiUrls import ClashApiV1Urls
from .DataDragonUrls import DataDragonUrls
from .LeagueApiUrls import LeagueApiV4Urls
from .LolStatusApiUrls import LolStatusApiV3Urls
from .MatchApiUrls import MatchApiV4Urls
from .SpectatorApiUrls import SpectatorApiV4Urls
from .SummonerApiUrls import SummonerApiV4Urls
from .ThirdPartyCodeApiUrls import ThirdPartyCodeApiV4Urls
from .MatchApiV5Urls import MatchApiV5Urls



from .ChampionApiV3 import ChampionApiV3
from .ChampionMasteryApiV4 import ChampionMasteryApiV4
from .ClashApiV1 import ClashApiV1
from .DataDragonApi import DataDragonApi
from .LeagueApiV4 import LeagueApiV4
from .LolStatusApiV3 import LolStatusApiV3
from .MatchApiV4 import MatchApiV4
from .SpectatorApiV4 import SpectatorApiV4
from .SummonerApiV4 import SummonerApiV4
from .ThirdPartyCodeApiV4 import ThirdPartyCodeApiV4
from .MatchApiV5 import MatchApiV5



from typing import Union
from .Deserializer import Deserializer
from .RateLimiter import RateLimiter

from .Handlers import (
    DeprecationHandler,
    DeserializerAdapter,
    DictionaryDeserializer,
    RateLimiterAdapter,
    ThrowOnErrorHandler,
    TypeCorrectorHandler,
)

from .Handlers.RateLimit import BasicRateLimiter

from ._apis import BaseApi, UrlConfig
from ._apis.league_of_legends import (
    ChampionApiV3,
    ChampionMasteryApiV4,
    ClashApiV1,
    DataDragonApi,
    LeagueApiV4,
    LolStatusApiV3,
    MatchApiV4,
    SpectatorApiV4,
    SummonerApiV4,
    MatchApiV5,
    ThirdPartyCodeApiV4,
)


class LolWatcher:
    """
    LolWatcher class is intended to be the main interaction point with the APIs for League of Legends.
    """

    def __init__(
        self,
        api_key: str = None,
        timeout: int = None,
        kernel_url: str = None,
        rate_limiter: RateLimiter = BasicRateLimiter(),
        deserializer: Deserializer = DictionaryDeserializer(),
        default_match_v5: bool = False,
    ):
        """
        Initialize a new instance of the RiotWatcher class.

        :param string api_key: the API key to use for this instance
        :param int timeout: Time to wait for a response before timing out a connection to
                            the Riot API
        :param string kernel_url: URL for the kernel instance to connect to, instead of the API.
                                  See https://github.com/meraki-analytics/kernel for details.
        :param RateLimiter rate_limiter: Instance to be used for rate limiting.
                                         This defaults to Handlers.RateLimit.BasicRateLimiter.
                                         This parameter is not used when connecting to a
                                         kernel instance.
        :param Deserializer deserializer: Instance to be used to deserialize responses
                                          from the Riot Api. Default is Handlers.DictionaryDeserializer.
        """
        if not kernel_url and not api_key:
            raise ValueError("Either api_key or kernel_url must be set!")

        if kernel_url:
            handler_chain = [
                DeserializerAdapter(deserializer),
                ThrowOnErrorHandler(),
                TypeCorrectorHandler(),
                DeprecationHandler(),
            ]
        else:
            handler_chain = [
                DeserializerAdapter(deserializer),
                ThrowOnErrorHandler(),
                TypeCorrectorHandler(),
                RateLimiterAdapter(rate_limiter),
                DeprecationHandler(),
            ]

        if kernel_url:
            UrlConfig.root_url = kernel_url
        else:
            UrlConfig.root_url = "https://{platform}.api.riotgames.com"

        self._base_api = BaseApi(api_key, handler_chain, timeout=timeout)

        self._champion = ChampionApiV3(self._base_api)
        self._lol_status = LolStatusApiV3(self._base_api)
        self._data_dragon = DataDragonApi(self._base_api)
        self._clash = ClashApiV1(self._base_api)
        self._champion_mastery = ChampionMasteryApiV4(self._base_api)
        self._league = LeagueApiV4(self._base_api)
        self._match_v4 = MatchApiV4(self._base_api)
        self._match_v5 = MatchApiV5(self._base_api)
        self._spectator = SpectatorApiV4(self._base_api)
        self._summoner = SummonerApiV4(self._base_api)
        self._third_party_code = ThirdPartyCodeApiV4(self._base_api)

        self._match = self._match_v5 if default_match_v5 else self._match_v4
        # todo: tournament-stub
        # todo: tournament

    @property
    def champion_mastery(self) -> ChampionMasteryApiV4:
        """
        Interface to the ChampionMastery Endpoint

        :rtype: league_of_legends.ChampionMasteryApiV4
        """
        return self._champion_mastery

    @property
    def champion(self) -> ChampionApiV3:
        """
        Interface to the Champion Endpoint

        :rtype: league_of_legends.ChampionApiV3
        """
        return self._champion

    @property
    def clash(self) -> ClashApiV1:
        """
        Interface to the Clash Endpoint

        :rtype: league_of_legends.ClashApiV1
        """
        return self._clash

    @property
    def league(self) -> LeagueApiV4:
        """
        Interface to the League Endpoint

        :rtype: league_of_legends.LeagueApiV4
        """
        return self._league

    @property
    def lol_status(self) -> LolStatusApiV3:
        """
        Interface to the LoLStatus Endpoint

        :rtype: league_of_legends.LolStatusApiV3
        """
        return self._lol_status

    @property
    def match(self) -> Union[MatchApiV4, MatchApiV5]:
        """
        Interface to the Match Endpoint

        :rtype: league_of_legends.MatchApiV5
        """
        return self._match

    @property
    def match_v4(self) -> MatchApiV4:
        """
        Temporary explicit interface to match-v4 endpoint.
        Will be removed when matchv4 is deprecated.

        :rtype: league_of_legends.MatchApiV4
        """
        return self._match

    @property
    def match_v5(self) -> MatchApiV5:
        """
        Temporary explicit interface to match-v5 endpoint.
        Will be removed when matchv4 is deprecated.

        :rtype: league_of_legends.MatchApiV5
        """
        return self._match_v5

    @property
    def spectator(self) -> SpectatorApiV4:
        """
        Interface to the Spectator Endpoint

        :rtype: league_of_legends.SpectatorApiV4
        """
        return self._spectator

    @property
    def data_dragon(self) -> DataDragonApi:
        """
        Interface to the DataDragon Endpoint

        :rtype: league_of_legends.DataDragonApi
        """
        return self._data_dragon

    @property
    def summoner(self) -> SummonerApiV4:
        """
        Interface to the Summoner Endpoint

        :rtype: league_of_legends.SummonerApiV4
        """
        return self._summoner

    @property
    def third_party_code(self) -> ThirdPartyCodeApiV4:
        """
        Interface to the Third Party Code Endpoint

        :rtype: league_of_legends.ThirdPartyCodeApiV4
        """
        return self._third_party_code



from setuptools import setup, find_packages
import os.path

here = os.path.dirname(__file__)

descr_file = os.path.join(here, "README.rst")
version_file = os.path.join(here, "src", "riotwatcher", "__version__.py")

version_info = {}
with open(version_file, "r") as f:
    exec(f.read(), version_info)

dev_requirements = ["coverage", "pre-commit", "pytest", "pytest-cov", "tox"]

setup(
    name=version_info["__title__"].lower(),
    version=version_info["__version__"],
    packages=find_packages("src", exclude=["test"]),
    package_dir={"": "src"},
    description=f"{version_info['__title__']} is a thin wrapper on top of the Riot Games API for League of Legends.",
    long_description=open(descr_file).read(),
    author="AG Stephan",
    url="https://github.com/pseudonym117/Riot-Watcher",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing",
        "Topic :: Games/Entertainment :: Real Time Strategy",
        "Topic :: Games/Entertainment :: Role-Playing",
    ],
    license="MIT",
    install_requires=["requests"],
    extras_require={"dev": dev_requirements},
)



from .Deserializer import Deserializer
from .RateLimiter import RateLimiter

from .Handlers import (
    DeprecationHandler,
    DeserializerAdapter,
    DictionaryDeserializer,
    RateLimiterAdapter,
    ThrowOnErrorHandler,
    TypeCorrectorHandler,
)
from .Handlers.RateLimit import BasicRateLimiter

from ._apis import BaseApi
from ._apis.riot import AccountApi


class RiotWatcher:
    """
    RiotWatcher class is intended to be the main interaction point with the generic Riot APIs.
    """

    def __init__(
        self,
        api_key: str,
        timeout: int = None,
        rate_limiter: RateLimiter = BasicRateLimiter(),
        deserializer: Deserializer = DictionaryDeserializer(),
    ):
        """
        Initialize a new instance of the RiotWatcher class.

        :param string api_key: the API key to use for this instance
        :param int timeout: Time to wait for a response before timing out a connection to
                            the Riot API
        :param RateLimiter rate_limiter: Instance to be used for rate limiting.
                                         This defaults to Handlers.RateLimit.BasicRateLimiter.
        :param Deserializer deserializer: Instance to be used to deserialize responses
                                          from the Riot Api. Default is Handlers.DictionaryDeserializer.
        """
        if not api_key:
            raise ValueError("api_key must be set!")

        handler_chain = [
            DeserializerAdapter(deserializer),
            ThrowOnErrorHandler(),
            TypeCorrectorHandler(),
            RateLimiterAdapter(rate_limiter),
            DeprecationHandler(),
        ]

        self._base_api = BaseApi(api_key, handler_chain, timeout=timeout)
        self._account = AccountApi(self._base_api)

    @property
    def account(self) -> AccountApi:
        """
        Interface to the Account Endpoint

        :rtype: riot.AccountApi
        """
        return self._account



from .ValEndpoint import ValEndpoint


class MatchEndpoint(ValEndpoint):
    def __init__(self, url: str, **kwargs):
        super().__init__(f"/match/v1{url}", **kwargs)


class MatchApiUrls:
    by_id = MatchEndpoint("/matches/{match_id}")
    matchlist_by_puuid = MatchEndpoint("/matchlists/by-puuid/{puuid}")
    recent_matches = MatchEndpoint("/recent-matches/by-queue/{queue}")



from .LeagueEndpoint import LeagueEndpoint


class LeagueV4Endpoint(LeagueEndpoint):
    def __init__(self, url, **kwargs):
        nurl = f"/league/v4{url}"
        super().__init__(nurl, **kwargs)


class LeagueApiV4Urls:
    challenger_by_queue = LeagueV4Endpoint("/challengerleagues/by-queue/{queue}")
    grandmaster_by_queue = LeagueV4Endpoint("/grandmasterleagues/by-queue/{queue}")
    by_id = LeagueV4Endpoint("/leagues/{league_id}")
    master_by_queue = LeagueV4Endpoint("/masterleagues/by-queue/{queue}")
    by_summoner = LeagueV4Endpoint("/entries/by-summoner/{encrypted_summoner_id}")
    entries = LeagueV4Endpoint("/entries/{queue}/{tier}/{division}", page=int)
